@@ -31,11 +31,56 @@ class PrismSession:
         logger.info("Starting new PRISM session...")
         self.singular_url = SINGLE_URL
         self.bulk_url = BULK_URL
-        self.download_dir = download_dir
-        self.driver_wait = driver_wait
+        if not isinstance(download_dir, str):
+            raise TypeError("download_dir must be a string")
+        else:
+            if os.path.isabs(download_dir):
+                self.download_dir = download_dir
+            else:
+                self.download_dir = os.path.abspath(download_dir)
+        if not isinstance(driver_wait, (int, float)):
+            raise TypeError("driver_wait must be an int or float")
+        else:
+            self.driver_wait = driver_wait
 
         chrome_options = Options()
         chrome_options.add_argument("--headless=new")
+        chrome_options.add_experimental_option(
+            "prefs",
+            {
+                "download.default_directory": self.download_dir,
+                "download.prompt_for_download": False,
+                "directory_upgrade": True,
+                "safebrowsing.enabled": True,
+            },
+        )
+
+        self.driver = webdriver.Chrome(
+            service=Service(ChromeDriverManager().install()), options=chrome_options
+        )
+
+        logger.info("PRISM session initialized.")
+
+    def close(self):
+        logger.info("Closing webdriver...")
+        self.driver.quit()
+        logger.info("Webdriver closed.")
+
+    def get_download_directory(self):
+        return self.download_dir
+
+    def set_download_directory(self, download_dir):
+        if not isinstance(download_dir, str):
+            raise TypeError("download_dir must be a string")
+        else:
+            if os.path.isabs(download_dir):
+                self.download_dir = download_dir
+            else:
+                self.download_dir = os.path.abspath(download_dir)
+        if self.driver:
+            self.close()
+        chrome_options = Options()
+        chrome_options.add_argument("--headless=new")
         chrome_options.add_experimental_option(
             "prefs",
             {
@@ -50,17 +95,18 @@ class PrismSession:
             service=Service(ChromeDriverManager().install()), options=chrome_options
         )
 
-        logger.info("PRISM session initialized.")
+    def get_driver_wait(self):
+        return self.driver_wait
 
-    def close(self):
-        logger.info("Closing PRISM session...")
-        self.driver.quit()
-        logger.info("PRISM session closed.")
+    def set_driver_wait(self, wait_time):
+        self.driver_wait = wait_time
 
     # WEATHER REQUESTS
     def get_30_year_monthly_normals(
         self,
         is_bulk_request: bool,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -80,8 +126,10 @@ class PrismSession:
         Retrieves PRISM baseline datasets describing average monthly and annual conditions over the most recent three full decades.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             precipitation (bool, optional): Whether to include precipitation data. Defaults to True.
             min_temp (bool, optional): Whether to include minimum temperature data. Defaults to False.
             mean_temp (bool, optional): Whether to include mean temperature data. Defaults to False.
@@ -103,6 +151,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -124,6 +174,8 @@ class PrismSession:
     def get_30_year_daily_normals(
         self,
         is_bulk_request: bool,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -139,8 +191,10 @@ class PrismSession:
         Retrieves PRISM baseline datasets describing average monthly and annual conditions over the most recent three full decades.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             precipitation (bool, optional): Whether to include precipitation data. Defaults to True.
             min_temp (bool, optional): Whether to include minimum temperature data. Defaults to False.
             mean_temp (bool, optional): Whether to include mean temperature data. Defaults to False.
@@ -158,6 +212,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -177,6 +233,8 @@ class PrismSession:
         is_bulk_request: bool,
         start_year,
         end_year,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -192,8 +250,10 @@ class PrismSession:
         Retrieves annual PRISM climate values for the specified coordinates and year range.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             start_year (int): Start year for the data range.
             end_year (int): End year for the data range.
             precipitation (bool, optional): Whether to include precipitation data. Defaults to True.
@@ -216,6 +276,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -238,6 +300,8 @@ class PrismSession:
         month,
         start_year,
         end_year,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -253,8 +317,10 @@ class PrismSession:
         Retrieves PRISM climate values for the given month for every year in the specified range for the specified coordinates.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             month (int): Month data to be retrieved for each year from start_year to end_year (inclusive).
             start_year (int): Year for the data.
             end_year (int): End year for the data.
@@ -282,6 +348,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -306,6 +374,8 @@ class PrismSession:
         start_year,
         end_month,
         end_year,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -321,8 +391,10 @@ class PrismSession:
         Retrieves monthly PRISM climate values for the specified coordinates and time range.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             start_month (int): Start month for the data range.
             start_year (int): Start year for the data range.
             end_month (int): End month for the data range.
@@ -347,6 +419,8 @@ class PrismSession:
             raise ValueError(
                 "Start month must be less than or equal to end month when years are equal."
             )
+        if not self._is_within_15_years(end_year, end_month, start_year, start_month):
+            raise ValueError("Monthly data requests are limited to a 15-year range.")
         if self._is_within_past_6_months(end_year, end_month, 1):
             warnings.warn(
                 "Data within past 6 months is provisional and may be subject to revision."
@@ -354,6 +428,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -379,6 +455,8 @@ class PrismSession:
         end_date,
         end_month,
         end_year,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=None,
         longitude=None,
         csv_path=None,
@@ -394,8 +472,10 @@ class PrismSession:
         Submits a request for daily climate data for the specified coordinates and time range.
 
         Args:
+            is_bulk_request (bool): Set to True if location input is .csv, False otherwise.
             latitude (float): Latitude of the location.
             longitude (float): Longitude of the location.
+            csv_path (str): Path to the CSV file containing coordinates data.
             start_date (int): Start date for the data range. Defaults to 1.
             start_month (int): Start month for the data range. Defaults to 1.
             start_year (int): Start year for the data range. Defaults to 2000.
@@ -430,6 +510,10 @@ class PrismSession:
             raise ValueError(
                 "Start date must be less than or equal to end date when months and years are equal."
             )
+        if not self._is_within_one_year(
+            start_year, start_month, start_date, end_year, end_month, end_date
+        ):
+            raise ValueError("Daily data requests are limited to a 1-year range.")
         if self._is_within_past_6_months(end_year, end_month, 1):
             warnings.warn(
                 "Data within past 6 months is provisional and may be subject to revision."
@@ -437,6 +521,8 @@ class PrismSession:
 
         self._submit_coordinates(
             is_bulk_request=is_bulk_request,
+            set_resolution_800m=set_resolution_800m,
+            set_units_metric=set_units_metric,
             latitude=latitude,
             longitude=longitude,
             csv_path=csv_path,
@@ -461,6 +547,8 @@ class PrismSession:
     def _submit_coordinates(
         self,
         is_bulk_request,
+        set_resolution_800m=False,
+        set_units_metric=False,
         latitude=40.9473,
         longitude=-112.2170,
         csv_path="/dummy/path/to/csv",
@@ -579,6 +667,13 @@ class PrismSession:
             solar_rad_clear_sky,
         )
 
+        # Set resolution to 4km or 800m grids
+        if set_resolution_800m:
+            self._set_resolution()
+
+        if set_units_metric:
+            self._set_units()
+
         # submit the form and download the data
         if is_bulk_request:
             if needs_partition:
@@ -602,6 +697,8 @@ class PrismSession:
                                 lon = float(row[1])
                         self._submit_coordinates(
                             is_bulk_request=False,
+                            set_resolution_800m=set_resolution_800m,
+                            set_units_metric=set_units_metric,
                             latitude=lat,
                             longitude=lon,
                             precipitation=precipitation,
@@ -807,6 +904,16 @@ class PrismSession:
                 )
                 button.click()
 
+    def _set_resolution(self):
+        WebDriverWait(self.driver, self.driver_wait).until(
+            EC.element_to_be_clickable((By.ID, "res_800m"))
+        ).click()
+
+    def _set_units(self):
+        WebDriverWait(self.driver, self.driver_wait).until(
+            EC.element_to_be_clickable((By.ID, "units_si"))
+        ).click()
+
     def _submit_and_download(self):
         WebDriverWait(self.driver, self.driver_wait).until(
             EC.element_to_be_clickable((By.ID, "submit_button"))
@@ -938,6 +1045,28 @@ class PrismSession:
         target = datetime.datetime(year, month, date)
         return target >= six_months_ago
 
+    def _is_within_15_years(self, start_year, start_month, end_year, end_month):
+        """
+        Method returns True if the start date is within 15 years of the end date. False otherwise.
+        """
+        start = datetime.datetime(start_year, start_month, 1)
+        end = datetime.datetime(end_year, end_month, 1)
+        diff_years = abs(end.year - start.year)
+        diff_months = abs(end.month - start.month)
+        total_months = diff_years * 12 + diff_months
+        return total_months <= 15 * 12
+
+    def _is_within_one_year(
+        self, start_year, start_month, start_day, end_year, end_month, end_day
+    ):
+        """
+        Method returns True if the start date is within one year of the end date. False otherwise.
+        """
+        start = datetime.datetime(start_year, start_month, start_day)
+        end = datetime.datetime(end_year, end_month, end_day)
+        diff_days = abs((end - start).days)
+        return diff_days <= 365
+
     def _is_string_float(self, value):
         try:
             float(value)
