@@ -11,6 +11,14 @@ import os
 def test_prism_session_init(mock_chrome):
     session = ps.PrismSession()
     assert isinstance(session.driver, MagicMock)
+    assert session.get_download_directory() == os.getcwd()
+    assert session.get_driver_wait() == 5
+    session = ps.PrismSession("./")
+    assert session.get_download_directory() == os.getcwd()
+    with pytest.raises(TypeError, match="download_dir must be a string"):
+        ps.PrismSession(download_dir=123)
+    with pytest.raises(TypeError, match="driver_wait must be an int or float"):
+        ps.PrismSession(driver_wait="five")
 
 
 @patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
@@ -20,8 +28,46 @@ def test_close(mock_chrome):
     session.driver.quit.assert_called_once()
 
 
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test_get_download_directory(mock_chrome):
+    session = ps.PrismSession()
+    download_dir = session.get_download_directory()
+    assert download_dir == os.getcwd()
+    session = ps.PrismSession("/dummy/directory")
+    assert session.get_download_directory() == "/dummy/directory"
+
+
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test_set_download_directory(mock_chrome):
+    session = ps.PrismSession()
+
+    session.set_download_directory("/new/download/dir")
+    assert session.get_download_directory() == "/new/download/dir"
+
+    session.set_download_directory("./")
+    assert session.get_download_directory() == os.getcwd()
+
+    with pytest.raises(TypeError, match="download_dir must be a string"):
+        session.set_download_directory(123)
+
+
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test_get_driver_wait(mock_chrome):
+    session = ps.PrismSession()
+    assert session.get_driver_wait() == 5
+
+
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test_set_driver_wait(mock_chrome):
+    session = ps.PrismSession()
+    session.set_driver_wait(10)
+    assert session.get_driver_wait() == 10
+
+
 @patch.object(ps.PrismSession, "_submit_and_download")
 @patch.object(ps.PrismSession, "_submit_and_download_bulk")
+@patch.object(ps.PrismSession, "_set_units")
+@patch.object(ps.PrismSession, "_set_resolution")
 @patch.object(ps.PrismSession, "_set_data_settings")
 @patch.object(ps.PrismSession, "_set_date_range")
 @patch.object(ps.PrismSession, "_set_coordinates")
@@ -35,6 +81,8 @@ def test_submit_coordinates(
     mock_set_coordinates,
     mock_set_date_range,
     mock_set_data_settings,
+    mock_set_resolution,
+    mock_set_units,
     mock_submit_and_download_bulk,
     mock_submit_and_download,
 ):
@@ -77,6 +125,24 @@ def test_submit_coordinates(
     assert mock_submit_and_download_bulk.call_count == 3
     mock_submit_and_download.assert_called_once()
 
+    # case where partition has 501 rows
+    session._submit_coordinates(
+        is_bulk_request=True, csv_path="tests/resources/single_row_partition.csv"
+    )
+
+    assert mock_validate_inputs.call_count == 5
+    assert mock_upload_csv.call_count == 4
+    assert mock_set_coordinates.call_count == 2
+    assert mock_set_date_range.call_count == 5
+    assert mock_set_data_settings.call_count == 5
+    assert mock_submit_and_download_bulk.call_count == 4
+    assert mock_submit_and_download.call_count == 2
+
+    session._submit_coordinates(is_bulk_request=False, set_resolution_800m=True, set_units_metric=True)
+
+    assert mock_set_resolution.call_count == 1
+    assert mock_set_units.call_count == 1
+
 
 @patch.object(ps.PrismSession, "_submit_coordinates", return_value=True)
 @patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
@@ -231,6 +297,19 @@ def test_get_monthly_values(mock_chrome, mock_submit_coordinates):
     )
     assert result is None
 
+    with pytest.raises(
+        ValueError, match="Monthly data requests are limited to a 15-year range."
+    ):
+        session.get_monthly_values(
+            is_bulk_request=False,
+            latitude=40.9473,
+            longitude=-112.2170,
+            start_month=1,
+            start_year=2000,
+            end_month=12,
+            end_year=2020,
+        )
+
     with pytest.raises(
         ValueError, match="Start year must be less than or equal to end year."
     ):
@@ -239,10 +318,25 @@ def test_get_monthly_values(mock_chrome, mock_submit_coordinates):
             latitude=40.9473,
             longitude=-112.2170,
             start_month=1,
-            start_year=3000,
+            start_year=2021,
             end_month=12,
             end_year=2020,
         )
+
+    with pytest.raises(
+        ValueError,
+        match="Start month must be less than or equal to end month when years are equal.",
+    ):
+        session.get_monthly_values(
+            is_bulk_request=False,
+            latitude=40.9473,
+            longitude=-112.2170,
+            start_month=12,
+            start_year=2020,
+            end_month=1,
+            end_year=2020,
+        )
+
     with pytest.warns(
         Warning,
         match="Data within past 6 months is provisional and may be subject to revision.",
@@ -300,11 +394,60 @@ def test_get_daily_values(mock_chrome, mock_submit_coordinates):
             longitude=-112.2170,
             start_date=31,
             start_month=12,
-            start_year=2027,
+            start_year=2021,
             end_date=1,
             end_month=1,
             end_year=2020,
         )
+
+    with pytest.raises(
+        ValueError,
+        match="Start month must be less than or equal to end month when years are equal.",
+    ):
+        session.get_daily_values(
+            is_bulk_request=False,
+            latitude=40.9473,
+            longitude=-112.2170,
+            start_date=31,
+            start_month=12,
+            start_year=2020,
+            end_date=1,
+            end_month=1,
+            end_year=2020,
+        )
+
+    with pytest.raises(
+        ValueError,
+        match="Start date must be less than or equal to end date when months and years are equal.",
+    ):
+        session.get_daily_values(
+            is_bulk_request=False,
+            latitude=40.9473,
+            longitude=-112.2170,
+            start_date=2,
+            start_month=1,
+            start_year=2020,
+            end_date=1,
+            end_month=1,
+            end_year=2020,
+        )
+
+    with pytest.raises(
+        ValueError,
+        match="Daily data requests are limited to a 1-year range.",
+    ):
+        session.get_daily_values(
+            is_bulk_request=False,
+            latitude=40.9473,
+            longitude=-112.2170,
+            start_date=1,
+            start_month=1,
+            start_year=2020,
+            end_date=1,
+            end_month=1,
+            end_year=2021,
+        )
+
     with pytest.warns(
         Warning,
         match="Data within past 6 months is provisional and may be subject to revision.",
@@ -315,7 +458,7 @@ def test_get_daily_values(mock_chrome, mock_submit_coordinates):
             longitude=-112.2170,
             start_date=31,
             start_month=12,
-            start_year=2020,
+            start_year=2023,
             end_date=1,
             end_month=12,
             end_year=2024,
@@ -573,6 +716,32 @@ def test__set_data_settings(mock_chrome, mock_wait):
     assert mock_element.click.call_count == 11
 
 
+@patch("src.prism_pull.prism_session.WebDriverWait")
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test__set_resolution(mock_chrome, mock_wait):
+    session = ps.PrismSession()
+    mock_element = MagicMock()
+    mock_element.click = MagicMock()
+    mock_wait.return_value.until.return_value = mock_element
+
+    session._set_resolution()
+
+    assert mock_element.click.call_count == 1
+
+
+@patch("src.prism_pull.prism_session.WebDriverWait")
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test__set_units(mock_chrome, mock_wait):
+    session = ps.PrismSession()
+    mock_element = MagicMock()
+    mock_element.click = MagicMock()
+    mock_wait.return_value.until.return_value = mock_element
+
+    session._set_units()
+
+    assert mock_element.click.call_count == 1
+
+
 @patch("src.prism_pull.prism_session.WebDriverWait")
 @patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
 def test__submit_and_download(mock_chrome, mock_wait):
@@ -760,3 +929,17 @@ def test__check_loc_and_download_type(mock_chrome):
         session._check_loc_and_download_type(
             "not_a_boolean", None, "not_a_number", "not_a_number"
         )
+
+
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test__is_within_15_years(mock_chrome):
+    session = ps.PrismSession()
+    assert session._is_within_15_years(2009, 1, 2025, 1) is False
+    assert session._is_within_15_years(2000, 1, 2010, 1) is True
+
+
+@patch("src.prism_pull.prism_session.webdriver.Chrome", return_value=MagicMock())
+def test__is_within_one_year(mock_chrome):
+    session = ps.PrismSession()
+    assert session._is_within_one_year(2024, 12, 31, 2025, 1, 1) is True
+    assert session._is_within_one_year(2024, 12, 31, 2026, 1, 1) is False
